<?php
/**
 * Plugin Name:BuddyPress Activity Comment Notifier
 * Plugin URI: http://buddydev.com/plugins/buddypress-activity-comment-notifier/
 * Author: Brajesh Singh
 * Author URI:http://buddydev.com/members/sbrajesh
 * Description: Show facebook like notification in the notification drop down when some user comment on your update or on other users update where you have commented
 * Version: 1.0.2
 * License: GPL
 * Site Wide Only: true
 * Network: true
 * Date Updated: 16th Feb, 2010
 * 
 */
/**
 * Special desclaimer: BuddyPress does not allows multiple items from same component with similar action(It is grouped by buddypress before delivering to the notfication formatting function), so I have hacked the component_action to be unique per item, because we want update for individual action items
 * Another hack is, buddypress activity format function is not declared as of buddypress 1.2.7/1.3 branch, so I am supplying a custom format function. The day it is added to buddypress, we will change it :)
 * Most important: It is just for fun :) hope you would like it
 */
define("BP_ACTIVITY_NOTIFIER_SLUG","ac_notification");
//register a dummy notifier component, I don't want to do it, but bp has no other mechanism for passing the notification data to function, so we need the format_notification_function
function bp_ac_notifier_setup_globals() {
	global $bp, $current_blog;

	/* Internal identifier */
	$bp->ac_notifier->id = 'ac_notifier';//I asume others are not going to use this is

	$bp->ac_notifier->slug = BP_ACTIVITY_NOTIFIER_SLUG;
	
	$bp->ac_notifier->format_notification_function = 'bp_ac_notifier_format_notifications';

	/* Register this in the active components array */
	$bp->active_components[$bp->ac_notifier->slug] = $bp->ac_notifier->id;

	do_action( 'bp_ac_notifier_setup_globals' );
}
add_action( 'bp_setup_globals', 'bp_ac_notifier_setup_globals' );

//storing notification for users

//notify all the users who have commented, or who was the original poster of the update, when someone comments
//hook to activity_comment_posted action
add_action("bp_activity_comment_posted","ac_notify_notify",10,2);
 function ac_notify_notify($comment_id, $params){
   global $bp;
   extract( $params );

   $users=ac_notify_find_involved_persons($activity_id);
   $activity=new BP_Activity_Activity($activity_id);
   //push the original poster user_id, if the original poster has not commented on his/her status yet
   if(!in_array($activity->user_id, $users)&&($bp->loggedin_user->id!=$activity->user_id))//if someone else is commenting
       array_push ($users, $activity->user_id);
   
    foreach((array)$users as $user_id){
               //create a notification
               bp_core_add_notification( $activity_id, $user_id, $bp->ac_notifier->id, 'new_activity_comment_'.$activity_id );//a hack to not allow grouping by component,action, rather group by component and individual action item
           }
  }

 //change in v 1.0.2, the name is changed
/** our notification format function which shows notification to user
 *
 * @global  $bp
 * @param <type> $action
 * @param <type> $activity_id
 * @param <type> $secondary_item_id
 * @param <type> $total_items
 * @return <type>
 */
function bp_ac_notifier_format_notifications( $action, $activity_id, $secondary_item_id, $total_items ) {
   
	global $bp;
        $user_names=array();
        $link=bp_activity_get_permalink( $activity_id );
        $activity = new BP_Activity_Activity( $activity_id );
       //if it is the original poster, say your, else say %s's post
        
        if($activity->user_id==$bp->loggedin_user->id){
                $text=__("your");
                $also="";
        }
        else{
          $text=sprintf(__("%s's"),  bp_core_get_user_displayname ($activity->user_id));//somone's
          $also=" also";
          
        }
        $ac_action='new_activity_comment_'.$activity_id;
     	if($action==$ac_action){
	//if ( (int)$total_items > 1 ) {
             $users=ac_notify_find_involved_persons($activity_id);
               $total_user= $count=count($users);//how many unique users have commented
                if($count>2){
                    $users=array_slice($users, $count-2);//just show name of two poster, rest should be as and 'n' other also commeted
                $count=$count-2;
                $glue=", ";
                }
                else if($total_user==2)
                    $glue=" and ";//if there are 2 unique users , say x and y commented

                foreach((array)$users as $user_id)
                    $user_names[]=bp_core_get_user_displayname ($user_id);
                
                if(!empty($user_names))
                     $commenting_users=join ($glue, $user_names);
                   
                
                if($total_user>2)
                    $text=$commenting_users." and ".$count." others".$also." commented on ".$text. " post";//can we change post to some meaningfull thing depending on the activity item ?
                else
                   $text=$commenting_users.$also ." commented on ".$text. " post";

                //sprintf( __('%s commented on your update' ),$commenting_users)
                return apply_filters( 'bp_activity_multiple_new_comment_notification', '<a href="' . $link. '">' . $text . '</a>');
		

	
        }
	return false;
}

 //for removing the notification if the single permalink screen is opened
add_action("bp_activity_screen_single_activity_permalink","ac_notify_remove_notification",10,2);
function ac_notify_remove_notification($activity,$has_access){
    global $bp;
if($has_access)//if user can view this activity, remove notification(just a safeguard for hidden activity)
    bp_core_delete_notifications_for_user_by_item_id(  $bp->loggedin_user->id, $activity->id, $bp->ac_notifier->id,  'new_activity_comment_'.$activity->id);
    
}
add_action("wp_head","bp_ac_hide_on_single_post");
function bp_ac_hide_on_single_post(){
    if(!(is_user_logged_in()&&  is_singular()))
        return;
    global $bp,$wpdb;
    $blog_id = (int)$wpdb->blogid;
  $post=wp_get_single_post();
   $activity_id=  bp_activity_get_activity_id( array(
		'user_id' => $post->post_author,
		'component' => $bp->blogs->id,
		'type' => "new_blog_post",
                "item_id"=>$blog_id,
		'secondary_item_id' => $post->ID
		
	));
  if(!empty($activity_id))
        bp_core_delete_notifications_for_user_by_item_id(  $bp->loggedin_user->id, $activity_id, $bp->ac_notifier->id,  'new_activity_comment_'.$activity_id);
  //now check for comments
  $activity_id='';
  $comments=bp_ac_get_all_comment_ids($post->ID);
  if(!empty($comments))
      foreach ($comments as $comment){
      //find activity id and delete the notification
      $activity_id=  bp_activity_get_activity_id( array(
		'user_id' => $comment->user_id,
		'component' => $bp->blogs->id,
		'type' => "new_blog_comment",
                "item_id"=>$blog_id,
		'secondary_item_id' => $comment->id

	));
      if(!empty($activity_id))
        bp_core_delete_notifications_for_user_by_item_id(  $bp->loggedin_user->id, $activity_id, $bp->ac_notifier->id,  'new_activity_comment_'.$activity_id);

  }
      
}

//delete notification for topic post view
////not implemented
//add_action("bp_before_group_forum_topic","ac_notifier_delete_notification_for_topic");
function ac_notifier_delete_notification_for_topic(){
global $bp;
$topic_slug = $bp->action_variables[1];
$topic_id = bp_forums_get_topic_id_from_slug( $topic_slug );
$group_id= $bp->groups->current_group->id;

$forum_id = groups_get_groupmeta($group_id, 'forum_id' );

//find all the posts in this topic

//find all the activities and delete, or may be we could have a better way by storing the secondary item id
//utwe may not have parent activity id?
                
    
}

//bp_core_delete_all_notifications_by_type($item_id, $component_name, $component_action);


//delete notification when an activity is deleted
//addd in v 1.0.2, thanks to @kat_uk for pointing the issue

add_action("bp_activity_deleted_activities","bp_ac_clear_notification_on_activity_delete");
function bp_ac_clear_notification_on_activity_delete($ac_ids){
    //we get an arry of activity ids
    global $bp;
 //bp_core_delete_notifications_for_user_by_item_id(  $bp->loggedin_user->id, $activity->id, $bp->activity->id,  'new_activity_comment_'.$activity->id);
 foreach((array)$ac_ids as $activity_id)
 bp_core_delete_all_notifications_by_type( $activity_id, $bp->ac_notifier->id, 'new_activity_comment_'.$activity_id, $secondary_item_id = false );
}


/*helper functions*/
/**
 *
 * @global <type> $bp
 * @global <type> $wpdb
 * @param <type> $activity_id
 * @return <type>
 * @desc: find all the unique user_id who have commented on this activity
 */

function ac_notify_find_involved_persons($activity_id){
    global $bp,$wpdb;
    $sql="select DISTINCT(user_id) from {$bp->activity->table_name} where item_id=%d and user_id!=%d ";
    $users=$wpdb->get_col($wpdb->prepare($sql,$activity_id,$bp->loggedin_user->id));//it finds all uses who commted on the activity
     return $users;
}
//for single post
//very bad thing to do here

function bp_ac_get_all_comment_ids($post_id) {
	global $wpdb;
	return $wpdb->get_results($wpdb->prepare("SELECT comment_ID as id,user_id FROM $wpdb->comments WHERE comment_post_ID = %d AND comment_approved = '1' ORDER BY comment_date", $post_id));
}

?>